=== modified file 'NEWS'
--- NEWS	2009-04-30 22:40:42 +0000
+++ NEWS	2009-05-06 05:36:28 +0000
@@ -32,6 +32,9 @@
 Improvements
 ************
 
+* ``--coverage`` works for code running in threads too.
+  (Andrew Bennets, Vincent Ladeuil)
+
 Bug Fixes
 *********
 
@@ -64,6 +67,9 @@
 * ssh usernames (defined in ~/.ssh/config) are honoured for bzr+ssh connections.
   (Vincent Ladeuil, #367726)
 
+* Several bugs related to unicode symlinks have been fixed and the test suite
+  enhanced to better catch regressions for them. (Vincent Ladeuil)
+
 Documentation
 *************
 
@@ -108,6 +114,9 @@
   refactoring to the core of clone allowing less round trips on new
   branches. (Robert Collins)
 
+* New method ``Tags.rename_revisions`` that can rename revision ids tags
+  are pointing at. (Jelmer Vernooij)
+
 * Updated the bundled ``ConfigObj`` library to 4.6.0 (Matt Nordhoff)
 
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2009-04-28 03:55:56 +0000
+++ bzrlib/branch.py	2009-05-06 05:36:28 +0000
@@ -855,12 +855,14 @@
         """
         raise NotImplementedError(self.pull)
 
-    def push(self, target, overwrite=False, stop_revision=None):
+    def push(self, target, overwrite=False, stop_revision=None, *args,
+        **kwargs):
         """Mirror this branch into target.
 
         This branch is considered to be 'local', having low latency.
         """
-        raise NotImplementedError(self.push)
+        return InterBranch.get(self, target).push(overwrite, stop_revision,
+            *args, **kwargs)
 
     def basis_tree(self):
         """Return `Tree` object for last revision."""
@@ -2228,75 +2230,10 @@
                 pass
         return None
 
-    @needs_read_lock
-    def push(self, target, overwrite=False, stop_revision=None,
-             _override_hook_source_branch=None):
-        """See Branch.push.
-
-        This is the basic concrete implementation of push()
-
-        :param _override_hook_source_branch: If specified, run
-        the hooks passing this Branch as the source, rather than self.
-        This is for use of RemoteBranch, where push is delegated to the
-        underlying vfs-based Branch.
-        """
-        # TODO: Public option to disable running hooks - should be trivial but
-        # needs tests.
-        return _run_with_write_locked_target(
-            target, self._push_with_bound_branches, target, overwrite,
-            stop_revision,
-            _override_hook_source_branch=_override_hook_source_branch)
-
-    def _push_with_bound_branches(self, target, overwrite,
-            stop_revision,
-            _override_hook_source_branch=None):
-        """Push from self into target, and into target's master if any.
-
-        This is on the base BzrBranch class even though it doesn't support
-        bound branches because the *target* might be bound.
-        """
-        def _run_hooks():
-            if _override_hook_source_branch:
-                result.source_branch = _override_hook_source_branch
-            for hook in Branch.hooks['post_push']:
-                hook(result)
-
-        bound_location = target.get_bound_location()
-        if bound_location and target.base != bound_location:
-            # there is a master branch.
-            #
-            # XXX: Why the second check?  Is it even supported for a branch to
-            # be bound to itself? -- mbp 20070507
-            master_branch = target.get_master_branch()
-            master_branch.lock_write()
-            try:
-                # push into the master from this branch.
-                self._basic_push(master_branch, overwrite, stop_revision)
-                # and push into the target branch from this. Note that we push from
-                # this branch again, because its considered the highest bandwidth
-                # repository.
-                result = self._basic_push(target, overwrite, stop_revision)
-                result.master_branch = master_branch
-                result.local_branch = target
-                _run_hooks()
-                return result
-            finally:
-                master_branch.unlock()
-        else:
-            # no master branch
-            result = self._basic_push(target, overwrite, stop_revision)
-            # TODO: Why set master_branch and local_branch if there's no
-            # binding?  Maybe cleaner to just leave them unset? -- mbp
-            # 20070504
-            result.master_branch = target
-            result.local_branch = None
-            _run_hooks()
-            return result
-
     def _basic_push(self, target, overwrite, stop_revision):
         """Basic implementation of push without bound branches or hooks.
 
-        Must be called with self read locked and target write locked.
+        Must be called with source read locked and target write locked.
         """
         result = BranchPushResult()
         result.source_branch = self
@@ -2307,10 +2244,11 @@
             # We assume that during 'push' this repository is closer than
             # the target.
             graph = self.repository.get_graph(target.repository)
-            target.update_revisions(self, stop_revision, overwrite=overwrite,
-                                    graph=graph)
+            target.update_revisions(self, stop_revision,
+                overwrite=overwrite, graph=graph)
         if self._push_should_merge_tags():
-            result.tag_conflicts = self.tags.merge_to(target.tags, overwrite)
+            result.tag_conflicts = self.tags.merge_to(target.tags,
+                overwrite)
         result.new_revno, result.new_revid = target.last_revision_info()
         return result
 
@@ -3016,6 +2954,14 @@
         """
         raise NotImplementedError(self.update_revisions)
 
+    def push(self, overwrite=False, stop_revision=None,
+             _override_hook_source_branch=None):
+        """Mirror the source branch into the target branch.
+
+        The source branch is considered to be 'local', having low latency.
+        """
+        raise NotImplementedError(self.push)
+
 
 class GenericInterBranch(InterBranch):
     """InterBranch implementation that uses public Branch functions.
@@ -3068,6 +3014,72 @@
         finally:
             self.source.unlock()
 
+    def push(self, overwrite=False, stop_revision=None,
+             _override_hook_source_branch=None):
+        """See InterBranch.push.
+
+        This is the basic concrete implementation of push()
+
+        :param _override_hook_source_branch: If specified, run
+        the hooks passing this Branch as the source, rather than self.
+        This is for use of RemoteBranch, where push is delegated to the
+        underlying vfs-based Branch.
+        """
+        # TODO: Public option to disable running hooks - should be trivial but
+        # needs tests.
+        self.source.lock_read()
+        try:
+            return _run_with_write_locked_target(
+                self.target, self._push_with_bound_branches, overwrite,
+                stop_revision,
+                _override_hook_source_branch=_override_hook_source_branch)
+        finally:
+            self.source.unlock()
+
+    def _push_with_bound_branches(self, overwrite, stop_revision,
+            _override_hook_source_branch=None):
+        """Push from source into target, and into target's master if any.
+        """
+        def _run_hooks():
+            if _override_hook_source_branch:
+                result.source_branch = _override_hook_source_branch
+            for hook in Branch.hooks['post_push']:
+                hook(result)
+
+        bound_location = self.target.get_bound_location()
+        if bound_location and self.target.base != bound_location:
+            # there is a master branch.
+            #
+            # XXX: Why the second check?  Is it even supported for a branch to
+            # be bound to itself? -- mbp 20070507
+            master_branch = self.target.get_master_branch()
+            master_branch.lock_write()
+            try:
+                # push into the master from the source branch.
+                self.source._basic_push(master_branch, overwrite, stop_revision)
+                # and push into the target branch from the source. Note that we
+                # push from the source branch again, because its considered the
+                # highest bandwidth repository.
+                result = self.source._basic_push(self.target, overwrite,
+                    stop_revision)
+                result.master_branch = master_branch
+                result.local_branch = self.target
+                _run_hooks()
+                return result
+            finally:
+                master_branch.unlock()
+        else:
+            # no master branch
+            result = self.source._basic_push(self.target, overwrite,
+                stop_revision)
+            # TODO: Why set master_branch and local_branch if there's no
+            # binding?  Maybe cleaner to just leave them unset? -- mbp
+            # 20070504
+            result.master_branch = self.target
+            result.local_branch = None
+            _run_hooks()
+            return result
+
     @classmethod
     def is_compatible(self, source, target):
         # GenericBranch uses the public API, so always compatible

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2009-03-30 11:49:32 +0000
+++ bzrlib/commands.py	2009-05-06 05:36:28 +0000
@@ -35,6 +35,7 @@
 lazy_import(globals(), """
 import codecs
 import errno
+import threading
 from warnings import warn
 
 import bzrlib
@@ -682,6 +683,7 @@
 
     tracer = trace.Trace(count=1, trace=0)
     sys.settrace(tracer.globaltrace)
+    threading.settrace(tracer.globaltrace)
 
     try:
         return exception_to_return_code(the_callable, *args, **kwargs)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2009-05-01 06:42:30 +0000
+++ bzrlib/errors.py	2009-05-06 05:36:28 +0000
@@ -1173,7 +1173,8 @@
 class DivergedBranches(BzrError):
 
     _fmt = ("These branches have diverged."
-            " Use the merge command to reconcile them.")
+            " Use the missing command to see how.\n"
+            "Use the merge command to reconcile them.")
 
     def __init__(self, branch1, branch2):
         self.branch1 = branch1

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2009-04-17 14:03:32 +0000
+++ bzrlib/hashcache.py	2009-05-06 05:36:28 +0000
@@ -180,7 +180,8 @@
                 filters = self._filter_provider(path=path, file_id=None)
             digest = self._really_sha1_file(abspath, filters)
         elif stat.S_ISLNK(mode):
-            digest = osutils.sha_string(os.readlink(abspath))
+            target = osutils.readlink(osutils.safe_unicode(abspath))
+            digest = osutils.sha_string(target.encode('UTF-8'))
         else:
             raise errors.BzrError("file %r: unknown file stat mode: %o"
                                   % (abspath, mode))

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2009-05-05 16:57:19 +0000
+++ bzrlib/osutils.py	2009-05-06 05:36:28 +0000
@@ -933,6 +933,20 @@
             and sys.platform not in ('cygwin', 'win32'))
 
 
+def readlink(abspath):
+    """Return a string representing the path to which the symbolic link points.
+
+    :param abspath: The link absolute unicode path.
+
+    This his guaranteed to return the symbolic link in unicode in all python
+    versions.
+    """
+    link = abspath.encode(_fs_enc)
+    target = os.readlink(link)
+    target = target.decode(_fs_enc)
+    return target
+
+
 def contains_whitespace(s):
     """True if there are any whitespace characters in s."""
     # string.whitespace can include '\xa0' in certain locales, because it is
@@ -1400,21 +1414,21 @@
             #       for win98 anyway.
             try:
                 from bzrlib._walkdirs_win32 import Win32ReadDir
-            except ImportError:
-                _selected_dir_reader = UnicodeDirReader()
-            else:
                 _selected_dir_reader = Win32ReadDir()
-        elif fs_encoding not in ('UTF-8', 'US-ASCII', 'ANSI_X3.4-1968'):
+            except ImportError:
+                pass
+        elif fs_encoding in ('UTF-8', 'US-ASCII', 'ANSI_X3.4-1968'):
             # ANSI_X3.4-1968 is a form of ASCII
-            _selected_dir_reader = UnicodeDirReader()
-        else:
             try:
                 from bzrlib._readdir_pyx import UTF8DirReader
-            except ImportError:
-                # No optimised code path
-                _selected_dir_reader = UnicodeDirReader()
-            else:
                 _selected_dir_reader = UTF8DirReader()
+            except ImportError:
+                pass
+
+    if _selected_dir_reader is None:
+        # Fallback to the python version
+        _selected_dir_reader = UnicodeDirReader()
+
     # 0 - relpath, 1- basename, 2- kind, 3- stat, 4-toppath
     # But we don't actually uses 1-3 in pending, so set them to None
     pending = [[_selected_dir_reader.top_prefix_to_starting_dir(top, prefix)]]

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2009-04-28 20:12:44 +0000
+++ bzrlib/reconfigure.py	2009-05-06 05:36:28 +0000
@@ -31,6 +31,7 @@
             self.repository = self.bzrdir.find_repository()
         except errors.NoRepositoryPresent:
             self.repository = None
+            self.local_repository = None
         else:
             if (self.repository.bzrdir.root_transport.base ==
                 self.bzrdir.root_transport.base):

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2009-04-09 20:23:07 +0000
+++ bzrlib/revisiontree.py	2009-05-06 05:36:28 +0000
@@ -125,7 +125,8 @@
 
     def get_symlink_target(self, file_id):
         ie = self._inventory[file_id]
-        return ie.symlink_target;
+        # Inventories store symlink targets in unicode
+        return ie.symlink_target
 
     def get_reference_revision(self, file_id, path=None):
         return self.inventory[file_id].reference_revision

=== modified file 'bzrlib/tag.py'
--- bzrlib/tag.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tag.py	2009-05-06 05:36:28 +0000
@@ -67,6 +67,10 @@
         # we never have anything to copy
         pass
 
+    def rename_revisions(self, rename_map):
+        # No tags, so nothing to rename
+        pass
+
     def get_reverse_tag_dict(self):
         # There aren't any tags, so the reverse mapping is empty.
         return {}
@@ -216,6 +220,17 @@
             to_tags.branch.unlock()
         return conflicts
 
+    def rename_revisions(self, rename_map):
+        """Rename revisions in this tags dictionary.
+        
+        :param rename_map: Dictionary mapping old revids to new revids
+        """
+        reverse_tags = self.get_reverse_tag_dict()
+        for revid, names in reverse_tags.iteritems():
+            if revid in rename_map:
+                for name in names:
+                    self.set_tag(name, rename_map[revid])
+
     def _reconcile_tags(self, source_dict, dest_dict, overwrite):
         """Do a two-way merge of two tag dictionaries.
 

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-05-01 03:50:47 +0000
+++ bzrlib/tests/__init__.py	2009-05-06 05:36:28 +0000
@@ -782,7 +782,7 @@
     _gather_lsprof_in_benchmarks = False
     attrs_to_keep = ('id', '_testMethodName', '_testMethodDoc',
                      '_log_contents', '_log_file_name', '_benchtime',
-                     '_TestCase__testMethodName')
+                     '_TestCase__testMethodName', '_TestCase__testMethodDoc',)
 
     def __init__(self, methodName='testMethod'):
         super(TestCase, self).__init__(methodName)
@@ -1446,11 +1446,9 @@
                 raise
         finally:
             saved_attrs = {}
-            absent_attr = object()
             for attr_name in self.attrs_to_keep:
-                attr = getattr(self, attr_name, absent_attr)
-                if attr is not absent_attr:
-                    saved_attrs[attr_name] = attr
+                if attr_name in self.__dict__:
+                    saved_attrs[attr_name] = self.__dict__[attr_name]
             self.__dict__ = saved_attrs
 
     def tearDown(self):

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2009-05-06 05:36:28 +0000
@@ -239,7 +239,8 @@
         out = self.run_bzr('pull ../branch_a', retcode=3)
         self.assertEqual(out,
                 ('','bzr: ERROR: These branches have diverged.'
-                    ' Use the merge command to reconcile them.\n'))
+                    ' Use the missing command to see how.\n'
+                    'Use the merge command to reconcile them.\n'))
         self.assertEqual(branch_b.get_parent(), parent)
         # test implicit --remember after resolving previous failure
         uncommit(branch=branch_b, tree=tree_b)

=== modified file 'bzrlib/tests/branch_implementations/test_sprout.py'
--- bzrlib/tests/branch_implementations/test_sprout.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/branch_implementations/test_sprout.py	2009-05-06 05:36:28 +0000
@@ -20,6 +20,7 @@
 from bzrlib import (
     branch as _mod_branch,
     errors,
+    osutils,
     remote,
     revision as _mod_revision,
     tests,
@@ -139,11 +140,20 @@
         # The link points to a file whose name is an omega
         # U+03A9 GREEK CAPITAL LETTER OMEGA
         # UTF-8: ce a9  UTF-16BE: 03a9  Decimal: &#937;
-        os.symlink(u'\u03a9','tree1/link_name')
-        tree.add(['link_name'],['link-id'])
+        target = u'\u03a9'
+        link_name = u'\N{Euro Sign}link'
+        os.symlink(target, 'tree1/' + link_name)
+        tree.add([link_name],['link-id'])
 
         revision = tree.commit('added a link to a Unicode target')
-        tree.bzrdir.sprout('target')
+        tree.bzrdir.sprout('dest')
+        self.assertEqual(target, osutils.readlink('dest/' + link_name))
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        # Check that the symlink target is safely round-tripped in the trees.
+        self.assertEqual(target, tree.get_symlink_target('link-id'))
+        self.assertEqual(target,
+                         tree.basis_tree().get_symlink_target('link-id'))
 
     def assertBranchHookBranchIsStacked(self, pre_change_params):
         # Just calling will either succeed or fail.

=== modified file 'bzrlib/tests/per_interbranch/__init__.py'
--- bzrlib/tests/per_interbranch/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_interbranch/__init__.py	2009-05-06 05:36:28 +0000
@@ -25,16 +25,27 @@
 """
 
 
+from bzrlib import (
+    branchbuilder,
+    memorytree,
+    )
 from bzrlib.branch import (
                            GenericInterBranch,
                            InterBranch,
                            )
+from bzrlib.bzrdir import (
+    BzrDirFormat,
+    BzrDirMetaFormat1,
+    )
 from bzrlib.errors import (
     FileExists,
+    NotBranchError,
     UninitializableFormat,
     )
-from bzrlib.tests import multiply_tests
-from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests import (
+    TestCaseWithTransport,
+    multiply_tests,
+    )
 from bzrlib.transport import get_transport
 
 
@@ -75,47 +86,69 @@
     return result
 
 
-class TestCaseWithInterBranch(TestCaseWithBzrDir):
-
-    def setUp(self):
-        super(TestCaseWithInterBranch, self).setUp()
-
-    def make_branch(self, relpath, format=None):
-        repo = self.make_repository(relpath, format=format)
-        return repo.bzrdir.create_branch()
-
-    def make_bzrdir(self, relpath, format=None):
-        try:
-            url = self.get_url(relpath)
-            segments = url.split('/')
-            if segments and segments[-1] not in ('', '.'):
-                parent = '/'.join(segments[:-1])
-                t = get_transport(parent)
-                try:
-                    t.mkdir(segments[-1])
-                except FileExists:
-                    pass
-            if format is None:
-                format = self.branch_format_from._matchingbzrdir
-            return format.initialize(url)
-        except UninitializableFormat:
-            raise TestSkipped("Format %s is not initializable." % format)
-
-    def make_repository(self, relpath, format=None):
-        made_control = self.make_bzrdir(relpath, format=format)
-        return made_control.create_repository()
-
-    def make_to_bzrdir(self, relpath):
-        return self.make_bzrdir(relpath,
-            self.branch_format_to._matchingbzrdir)
+class TestCaseWithInterBranch(TestCaseWithTransport):
+
+    def make_from_branch(self, relpath):
+        repo = self.make_repository(relpath)
+        return self.branch_format_from.initialize(repo.bzrdir)
+
+    def make_from_branch_and_memory_tree(self, relpath):
+        """Create a branch on the default transport and a MemoryTree for it."""
+        b = self.make_from_branch(relpath)
+        return memorytree.MemoryTree.create_on_branch(b)
+
+    def make_from_branch_and_tree(self, relpath):
+        """Create a branch on the default transport and a working tree for it."""
+        b = self.make_from_branch(relpath)
+        return b.bzrdir.create_workingtree()
+
+    def make_from_branch_builder(self, relpath):
+        default_format = BzrDirFormat.get_default_format()
+        format = BzrDirMetaFormat1()
+        format.set_branch_format(self.branch_format_from)
+        format.repository_format = default_format.repository_format
+        format.workingtree_format = default_format.workingtree_format
+        return branchbuilder.BranchBuilder(self.get_transport(relpath),
+            format=format)
 
     def make_to_branch(self, relpath):
-        made_control = self.make_to_bzrdir(relpath)
-        return self.branch_format_to.initialize(made_control)
+        repo = self.make_repository(relpath)
+        return self.branch_format_to.initialize(repo.bzrdir)
+
+    def make_to_branch_and_memory_tree(self, relpath):
+        """Create a branch on the default transport and a MemoryTree for it."""
+        b = self.make_to_branch(relpath)
+        return memorytree.MemoryTree.create_on_branch(b)
+
+    def make_to_branch_and_tree(self, relpath):
+        """Create a branch on the default transport and a working tree for it."""
+        b = self.make_to_branch(relpath)
+        return b.bzrdir.create_workingtree()
+
+    def sprout_to(self, origdir, to_url):
+        """Sprout a bzrdir, using to_format for the new branch."""
+        newbranch = self.make_to_branch(to_url)
+        origbranch = origdir.open_branch()
+        newbranch.repository.fetch(origbranch.repository)
+        origbranch.copy_content_into(newbranch)
+        newbranch.bzrdir.create_workingtree()
+        return newbranch.bzrdir
+
+    def sprout_from(self, origdir, to_url):
+        """Sprout a bzrdir, using from_format for the new bzrdir."""
+        newbranch = self.make_from_branch(to_url)
+        origbranch = origdir.open_branch()
+        newbranch.repository.fetch(origbranch.repository)
+        origbranch.copy_content_into(newbranch)
+        newbranch.bzrdir.create_workingtree()
+        return newbranch.bzrdir
+
+
 
 
 def load_tests(standard_tests, module, loader):
     submod_tests = loader.loadTestsFromModuleNames([
+        'bzrlib.tests.per_interbranch.test_push',
         'bzrlib.tests.per_interbranch.test_update_revisions',
         ])
     scenarios = make_scenarios(default_test_list())

=== added file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2009-05-05 12:54:51 +0000
@@ -0,0 +1,379 @@
+# Copyright (C) 2004, 2005, 2007, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for branch.push behaviour."""
+
+from cStringIO import StringIO
+import os
+
+from bzrlib import (
+    branch,
+    builtins,
+    bzrdir,
+    debug,
+    errors,
+    push,
+    repository,
+    tests,
+    )
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.memorytree import MemoryTree
+from bzrlib.revision import NULL_REVISION
+from bzrlib.smart import client, server
+from bzrlib.smart.repository import SmartServerRepositoryGetParentMap
+from bzrlib.tests.per_interbranch import (
+    TestCaseWithInterBranch,
+    )
+from bzrlib.transport import get_transport
+from bzrlib.transport.local import LocalURLServer
+
+
+# These tests are based on similar tests in 
+# bzrlib.tests.branch_implementations.test_push.
+
+
+class TestPush(TestCaseWithInterBranch):
+
+    def test_push_convergence_simple(self):
+        # when revisions are pushed, the left-most accessible parents must
+        # become the revision-history.
+        mine = self.make_from_branch_and_tree('mine')
+        mine.commit('1st post', rev_id='P1', allow_pointless=True)
+        other = self.sprout_to(mine.bzrdir, 'other').open_workingtree()
+        other.commit('my change', rev_id='M1', allow_pointless=True)
+        mine.merge_from_branch(other.branch)
+        mine.commit('merge my change', rev_id='P2')
+        result = mine.branch.push(other.branch)
+        self.assertEqual(['P1', 'P2'], other.branch.revision_history())
+        # result object contains some structured data
+        self.assertEqual(result.old_revid, 'M1')
+        self.assertEqual(result.new_revid, 'P2')
+        # and it can be treated as an integer for compatibility
+        self.assertEqual(int(result), 0)
+
+    def test_push_merged_indirect(self):
+        # it should be possible to do a push from one branch into another
+        # when the tip of the target was merged into the source branch
+        # via a third branch - so its buried in the ancestry and is not
+        # directly accessible.
+        mine = self.make_from_branch_and_tree('mine')
+        mine.commit('1st post', rev_id='P1', allow_pointless=True)
+        target = self.sprout_to(mine.bzrdir, 'target').open_workingtree()
+        target.commit('my change', rev_id='M1', allow_pointless=True)
+        other = self.sprout_to(mine.bzrdir, 'other').open_workingtree()
+        other.merge_from_branch(target.branch)
+        other.commit('merge my change', rev_id='O2')
+        mine.merge_from_branch(other.branch)
+        mine.commit('merge other', rev_id='P2')
+        mine.branch.push(target.branch)
+        self.assertEqual(['P1', 'P2'], target.branch.revision_history())
+
+    def test_push_to_checkout_updates_master(self):
+        """Pushing into a checkout updates the checkout and the master branch"""
+        master_tree = self.make_to_branch_and_tree('master')
+        checkout = self.make_to_branch_and_tree('checkout')
+        try:
+            checkout.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        rev1 = checkout.commit('master')
+
+        other_bzrdir = self.sprout_from(master_tree.branch.bzrdir, 'other')
+        other = other_bzrdir.open_workingtree()
+        rev2 = other.commit('other commit')
+        # now push, which should update both checkout and master.
+        other.branch.push(checkout.branch)
+        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
+        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+
+    def test_push_raises_specific_error_on_master_connection_error(self):
+        master_tree = self.make_to_branch_and_tree('master')
+        checkout = self.make_to_branch_and_tree('checkout')
+        try:
+            checkout.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        other_bzrdir = self.sprout_from(master_tree.branch.bzrdir, 'other')
+        other = other_bzrdir.open_workingtree()
+        # move the branch out of the way on disk to cause a connection
+        # error.
+        os.rename('master', 'master_gone')
+        # try to push, which should raise a BoundBranchConnectionFailure.
+        self.assertRaises(errors.BoundBranchConnectionFailure,
+                other.branch.push, checkout.branch)
+
+    def test_push_uses_read_lock(self):
+        """Push should only need a read lock on the source side."""
+        source = self.make_from_branch_and_tree('source')
+        target = self.make_to_branch('target')
+
+        self.build_tree(['source/a'])
+        source.add(['a'])
+        source.commit('a')
+
+        source.branch.lock_read()
+        try:
+            target.lock_write()
+            try:
+                source.branch.push(target, stop_revision=source.last_revision())
+            finally:
+                target.unlock()
+        finally:
+            source.branch.unlock()
+
+    def test_push_within_repository(self):
+        """Push from one branch to another inside the same repository."""
+        try:
+            repo = self.make_repository('repo', shared=True)
+        except (errors.IncompatibleFormat, errors.UninitializableFormat):
+            # This Branch format cannot create shared repositories
+            return
+        # This is a little bit trickier because make_branch_and_tree will not
+        # re-use a shared repository.
+        try:
+            a_branch = self.make_from_branch('repo/tree')
+        except (errors.UninitializableFormat):
+            # Cannot create these branches
+            return
+        try:
+            tree = a_branch.bzrdir.create_workingtree()
+        except errors.NotLocalUrl:
+            if self.vfs_transport_factory is LocalURLServer:
+                # the branch is colocated on disk, we cannot create a checkout.
+                # hopefully callers will expect this.
+                local_controldir = bzrdir.BzrDir.open(self.get_vfs_only_url('repo/tree'))
+                tree = local_controldir.create_workingtree()
+            else:
+                tree = a_branch.create_checkout('repo/tree', lightweight=True)
+        self.build_tree(['repo/tree/a'])
+        tree.add(['a'])
+        tree.commit('a')
+
+        to_branch = self.make_to_branch('repo/branch')
+        tree.branch.push(to_branch)
+
+        self.assertEqual(tree.branch.last_revision(),
+                         to_branch.last_revision())
+
+    def test_push_overwrite_of_non_tip_with_stop_revision(self):
+        """Combining the stop_revision and overwrite options works.
+
+        This was <https://bugs.launchpad.net/bzr/+bug/234229>.
+        """
+        source = self.make_from_branch_and_tree('source')
+        target = self.make_to_branch('target')
+
+        source.commit('1st commit')
+        source.branch.push(target)
+        source.commit('2nd commit', rev_id='rev-2')
+        source.commit('3rd commit')
+
+        source.branch.push(target, stop_revision='rev-2', overwrite=True)
+        self.assertEqual('rev-2', target.last_revision())
+
+    def test_push_with_default_stacking_does_not_create_broken_branch(self):
+        """Pushing a new standalone branch works even when there's a default
+        stacking policy at the destination.
+
+        The new branch will preserve the repo format (even if it isn't the
+        default for the branch), and will be stacked when the repo format
+        allows (which means that the branch format isn't necessarly preserved).
+        """
+        if isinstance(self.branch_format_from, branch.BzrBranchFormat4):
+            raise tests.TestNotApplicable('Not a metadir format.')
+        if isinstance(self.branch_format_from, branch.BranchReferenceFormat):
+            # This test could in principle apply to BranchReferenceFormat, but
+            # make_branch_builder doesn't support it.
+            raise tests.TestSkipped(
+                "BranchBuilder can't make reference branches.")
+        # Make a branch called "local" in a stackable repository
+        # The branch has 3 revisions:
+        #   - rev-1, adds a file
+        #   - rev-2, no changes
+        #   - rev-3, modifies the file.
+        repo = self.make_repository('repo', shared=True, format='1.6')
+        builder = self.make_from_branch_builder('repo/local')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('filename', 'f-id', 'file', 'content\n'))])
+        builder.build_snapshot('rev-2', ['rev-1'], [])
+        builder.build_snapshot('rev-3', ['rev-2'],
+            [('modify', ('f-id', 'new-content\n'))])
+        builder.finish_series()
+        trunk = builder.get_branch()
+        # Sprout rev-1 to "trunk", so that we can stack on it.
+        trunk.bzrdir.sprout(self.get_url('trunk'), revision_id='rev-1')
+        # Set a default stacking policy so that new branches will automatically
+        # stack on trunk.
+        self.make_bzrdir('.').get_config().set_default_stack_on('trunk')
+        # Push rev-2 to a new branch "remote".  It will be stacked on "trunk".
+        output = StringIO()
+        push._show_push_branch(trunk, 'rev-2', self.get_url('remote'), output)
+        # Push rev-3 onto "remote".  If "remote" not stacked and is missing the
+        # fulltext record for f-id @ rev-1, then this will fail.
+        remote_branch = Branch.open(self.get_url('remote'))
+        trunk.push(remote_branch)
+        remote_branch.check()
+
+    def test_no_get_parent_map_after_insert_stream(self):
+        # Effort test for bug 331823
+        self.setup_smart_server_with_call_log()
+        # Make a local branch with four revisions.  Four revisions because:
+        # one to push, one there for _walk_to_common_revisions to find, one we
+        # don't want to access, one for luck :)
+        if isinstance(self.branch_format_from, branch.BranchReferenceFormat):
+            # This test could in principle apply to BranchReferenceFormat, but
+            # make_branch_builder doesn't support it.
+            raise tests.TestSkipped(
+                "BranchBuilder can't make reference branches.")
+        try:
+            builder = self.make_from_branch_builder('local')
+        except (errors.TransportNotPossible, errors.UninitializableFormat):
+            raise tests.TestNotApplicable('format not directly constructable')
+        builder.start_series()
+        builder.build_snapshot('first', None, [
+            ('add', ('', 'root-id', 'directory', ''))])
+        builder.build_snapshot('second', ['first'], [])
+        builder.build_snapshot('third', ['second'], [])
+        builder.build_snapshot('fourth', ['third'], [])
+        builder.finish_series()
+        local = builder.get_branch()
+        local = branch.Branch.open(self.get_vfs_only_url('local'))
+        # Initial push of three revisions
+        remote_bzrdir = local.bzrdir.sprout(
+            self.get_url('remote'), revision_id='third')
+        remote = remote_bzrdir.open_branch()
+        # Push fourth revision
+        self.reset_smart_call_log()
+        self.disableOptimisticGetParentMap()
+        self.assertFalse(local.is_locked())
+        local.push(remote)
+        hpss_call_names = [item.call.method for item in self.hpss_calls]
+        self.assertTrue('Repository.insert_stream' in hpss_call_names)
+        insert_stream_idx = hpss_call_names.index('Repository.insert_stream')
+        calls_after_insert_stream = hpss_call_names[insert_stream_idx:]
+        # After inserting the stream the client has no reason to query the
+        # remote graph any further.
+        self.assertEqual(
+            ['Repository.insert_stream', 'Repository.insert_stream', 'get',
+             'Branch.set_last_revision_info', 'Branch.unlock'],
+            calls_after_insert_stream)
+
+    def disableOptimisticGetParentMap(self):
+        # Tweak some class variables to stop remote get_parent_map calls asking
+        # for or receiving more data than the caller asked for.
+        old_flag = SmartServerRepositoryGetParentMap.no_extra_results
+        inter_class = repository.InterRepository
+        old_batch_size = inter_class._walk_to_common_revisions_batch_size
+        inter_class._walk_to_common_revisions_batch_size = 1
+        SmartServerRepositoryGetParentMap.no_extra_results = True
+        def reset_values():
+            SmartServerRepositoryGetParentMap.no_extra_results = old_flag
+            inter_class._walk_to_common_revisions_batch_size = old_batch_size
+        self.addCleanup(reset_values)
+
+
+class TestPushHook(TestCaseWithInterBranch):
+
+    def setUp(self):
+        self.hook_calls = []
+        TestCaseWithInterBranch.setUp(self)
+
+    def capture_post_push_hook(self, result):
+        """Capture post push hook calls to self.hook_calls.
+
+        The call is logged, as is some state of the two branches.
+        """
+        if result.local_branch:
+            local_locked = result.local_branch.is_locked()
+            local_base = result.local_branch.base
+        else:
+            local_locked = None
+            local_base = None
+        self.hook_calls.append(
+            ('post_push', result.source_branch, local_base,
+             result.master_branch.base,
+             result.old_revno, result.old_revid,
+             result.new_revno, result.new_revid,
+             result.source_branch.is_locked(), local_locked,
+             result.master_branch.is_locked()))
+
+    def test_post_push_empty_history(self):
+        target = self.make_to_branch('target')
+        source = self.make_from_branch('source')
+        Branch.hooks.install_named_hook('post_push',
+                                        self.capture_post_push_hook, None)
+        source.push(target)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_push', source, None, target.base, 0, NULL_REVISION,
+             0, NULL_REVISION, True, None, True)
+            ],
+            self.hook_calls)
+
+    def test_post_push_bound_branch(self):
+        # pushing to a bound branch should pass in the master branch to the
+        # hook, allowing the correct number of emails to be sent, while still
+        # allowing hooks that want to modify the target to do so to both
+        # instances.
+        target = self.make_to_branch('target')
+        local = self.make_from_branch('local')
+        try:
+            local.bind(target)
+        except errors.UpgradeRequired:
+            # We can't bind this format to itself- typically it is the local
+            # branch that doesn't support binding.  As of May 2007
+            # remotebranches can't be bound.  Let's instead make a new local
+            # branch of the default type, which does allow binding.
+            # See https://bugs.launchpad.net/bzr/+bug/112020
+            local = BzrDir.create_branch_convenience('local2')
+            local.bind(target)
+        source = self.make_from_branch('source')
+        Branch.hooks.install_named_hook('post_push',
+                                        self.capture_post_push_hook, None)
+        source.push(local)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_push', source, local.base, target.base, 0, NULL_REVISION,
+             0, NULL_REVISION, True, True, True)
+            ],
+            self.hook_calls)
+
+    def test_post_push_nonempty_history(self):
+        target = self.make_to_branch_and_tree('target')
+        target.lock_write()
+        target.add('')
+        rev1 = target.commit('rev 1')
+        target.unlock()
+        sourcedir = target.bzrdir.clone(self.get_url('source'))
+        source = MemoryTree.create_on_branch(sourcedir.open_branch())
+        rev2 = source.commit('rev 2')
+        Branch.hooks.install_named_hook('post_push',
+                                        self.capture_post_push_hook, None)
+        source.branch.push(target.branch)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_push', source.branch, None, target.branch.base, 1, rev1,
+             2, rev2, True, None, True)
+            ],
+            self.hook_calls)

=== modified file 'bzrlib/tests/per_interbranch/test_update_revisions.py'
--- bzrlib/tests/per_interbranch/test_update_revisions.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_interbranch/test_update_revisions.py	2009-05-06 05:36:28 +0000
@@ -28,10 +28,10 @@
         super(TestUpdateRevisions, self).setUp()
         self.tree1 = self.make_branch_and_tree('tree1')
         rev1 = self.tree1.commit('one')
-        dir2 = self.make_to_bzrdir('tree2')
-        dir2.create_repository().fetch(self.tree1.branch.repository)
-        branch2 = self.tree1.branch.sprout(dir2)
-        self.tree2 = dir2.create_workingtree()
+        branch2 = self.make_to_branch('tree2')
+        branch2.repository.fetch(self.tree1.branch.repository)
+        self.tree1.branch.copy_content_into(branch2)
+        self.tree2 = branch2.bzrdir.create_workingtree()
 
     def test_accepts_graph(self):
         # An implementation may not use it, but it should allow a 'graph' to be

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2009-04-09 20:23:07 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2009-05-06 05:36:28 +0000
@@ -735,9 +735,12 @@
             mini_commit=self.mini_commit_record_iter_changes)
 
     def _add_commit_change_check_changed(self, tree, name, changer,
-        expect_fs_hash=False, mini_commit=None):
-        tree.add([name], [name + 'id'])
-        self._commit_change_check_changed(tree, name, name + 'id',
+        expect_fs_hash=False, mini_commit=None, file_id=None):
+        if file_id is None:
+            file_id = name + 'id'
+        tree.add([name], [file_id])
+        self._commit_change_check_changed(
+            tree, name, file_id,
             changer, expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
 
     def _commit_change_check_changed(self, tree, name, file_id, changer,
@@ -947,16 +950,32 @@
             os.symlink('newtarget', 'link')
         self._add_commit_change_check_changed(tree, 'link', change_link)
 
-    def test_last_modified_revision_after_content_link_changes_ric(self):
+    def _test_last_mod_rev_after_content_link_changes_ric(
+        self, link, target, newtarget, file_id=None):
+        if file_id is None:
+            file_id = link
         # changing a link changes the last modified.
         self.requireFeature(tests.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
-        os.symlink('target', 'link')
+        os.symlink(target, link)
         def change_link():
-            os.unlink('link')
-            os.symlink('newtarget', 'link')
-        self._add_commit_change_check_changed(tree, 'link', change_link,
-            mini_commit=self.mini_commit_record_iter_changes)
+            os.unlink(link)
+            os.symlink(newtarget, link)
+        self._add_commit_change_check_changed(
+            tree, link, change_link,
+            mini_commit=self.mini_commit_record_iter_changes,
+            file_id=file_id)
+
+    def test_last_modified_rev_after_content_link_changes_ric(self):
+        self._test_last_mod_rev_after_content_link_changes_ric(
+            'link', 'target', 'newtarget')
+
+    def test_last_modified_rev_after_content_unicode_link_changes_ric(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        self._test_last_mod_rev_after_content_link_changes_ric(
+            u'li\u1234nk', u'targ\N{Euro Sign}t', u'n\N{Euro Sign}wtarget',
+
+            file_id=u'li\u1234nk'.encode('UTF-8'))
 
     def _commit_sprout(self, tree, name):
         tree.add([name], [name + 'id'])

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2009-05-06 05:36:28 +0000
@@ -27,25 +27,68 @@
     tests,
     )
 from bzrlib.tests import (
-        SymlinkFeature,
-        )
-from bzrlib.tests import test_dirstate
+    test_dirstate,
+    test_osutils,
+    )
+
+try:
+    from bzrlib import _dirstate_helpers_c
+    has_dirstate_helpers_c = True
+except ImportError:
+    has_dirstate_helpers_c = False
 
 
 class _CompiledDirstateHelpersFeature(tests.Feature):
     def _probe(self):
-        try:
-            import bzrlib._dirstate_helpers_c
-        except ImportError:
-            return False
-        return True
+        return has_dirstate_helpers_c
 
     def feature_name(self):
         return 'bzrlib._dirstate_helpers_c'
-
 CompiledDirstateHelpersFeature = _CompiledDirstateHelpersFeature()
 
 
+def load_tests(basic_tests, module, loader):
+    # FIXME: we should also parametrize against SHA1Provider !
+    suite = loader.suiteClass()
+    remaining_tests = basic_tests
+
+    dir_reader_scenarios = test_osutils.dir_reader_scenarios()
+
+    ue_scenarios = [('dirstate_Python',
+                     {'update_entry': dirstate.py_update_entry})]
+    if has_dirstate_helpers_c:
+        c_scenario = ('dirstate_C',
+                     {'update_entry': _dirstate_helpers_c.update_entry})
+        ue_scenarios.append(c_scenario)
+    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance(TestUpdateEntry))
+    tests.multiply_tests(process_entry_tests,
+                         tests.multiply_scenarios(dir_reader_scenarios,
+                                                  ue_scenarios),
+                         suite)
+
+    pe_scenarios = [('dirstate_Python',
+                     {'_process_entry': dirstate.ProcessEntryPython})]
+    if has_dirstate_helpers_c:
+        c_scenario = ('dirstate_C',
+                     {'_process_entry': _dirstate_helpers_c.ProcessEntryC})
+        pe_scenarios.append(c_scenario)
+    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance(TestProcessEntry))
+    tests.multiply_tests(process_entry_tests,
+                         tests.multiply_scenarios(dir_reader_scenarios,
+                                                  pe_scenarios),
+                         suite)
+
+    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance(
+            test_dirstate.TestCaseWithDirState))
+    tests.multiply_tests(dir_reader_tests, dir_reader_scenarios, suite)
+    suite.addTest(remaining_tests)
+
+    return suite
+
+
 class TestBisectPathMixin(object):
     """Test that _bisect_path_*() returns the expected values.
 
@@ -790,18 +833,25 @@
 class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
     """Test the DirState.update_entry functions"""
 
+    # Set by load_tests
+    update_entry = None
+
+    def setUp(self):
+        super(TestUpdateEntry, self).setUp()
+        orig = dirstate.update_entry
+        def cleanup():
+            dirstate.update_entry = orig
+        self.addCleanup(cleanup)
+        dirstate.update_entry = self.update_entry
+
     def get_state_with_a(self):
         """Create a DirState tracking a single object named 'a'"""
         state = test_dirstate.InstrumentedDirState.initialize('dirstate')
         self.addCleanup(state.unlock)
         state.add('a', 'a-id', 'file', None, '')
         entry = state._get_entry(0, path_utf8='a')
-        self.set_update_entry()
         return state, entry
 
-    def set_update_entry(self):
-        self.update_entry = dirstate.py_update_entry
-
     def test_observed_sha1_cachable(self):
         state, entry = self.get_state_with_a()
         atime = time.time() - 10
@@ -921,7 +971,7 @@
 
     def test_update_entry_symlink(self):
         """Update entry should read symlinks."""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         state.save()
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
@@ -1022,7 +1072,6 @@
                          state._dirblock_state)
 
     def test_update_entry_tree_reference(self):
-        self.set_update_entry()
         state = test_dirstate.InstrumentedDirState.initialize('dirstate')
         self.addCleanup(state.unlock)
         state.add('r', 'r-id', 'tree-reference', None, '')
@@ -1064,6 +1113,7 @@
 
         return packed_stat
 
+    # FIXME: Add unicode version
     def create_and_test_symlink(self, state, entry):
         """Create a symlink at 'a' and verify the state finds it.
 
@@ -1098,7 +1148,7 @@
 
     def test_update_file_to_symlink(self):
         """File becomes a symlink"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The file sha1 won't be cached unless the file is old
         state.adjust_time(+10)
@@ -1117,7 +1167,7 @@
 
     def test_update_dir_to_symlink(self):
         """Directory becomes a symlink"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1127,7 +1177,7 @@
 
     def test_update_symlink_to_file(self):
         """Symlink becomes a file"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink and file info won't be cached unless old
         state.adjust_time(+10)
@@ -1137,7 +1187,7 @@
 
     def test_update_symlink_to_dir(self):
         """Symlink becomes a directory"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1166,8 +1216,8 @@
         self.assertEqual([('f', '', 14, True, dirstate.DirState.NULLSTAT)],
                          entry[1])
 
-        # Make the disk object look old enough to cache (but it won't cache the sha
-        # as it is a new file).
+        # Make the disk object look old enough to cache (but it won't cache the
+        # sha as it is a new file).
         state.adjust_time(+20)
         digest = 'b50e5406bb5e153ebbeb20268fcf37c87e1ecfb6'
         self.update_entry(state, entry, abspath='a', stat_value=stat_value)
@@ -1194,8 +1244,8 @@
         entry = state._get_entry(0, path_utf8='a file')
         state._sha_cutoff_time()
         state._cutoff_time += 10
-        sha1 = dirstate.update_entry(state, entry, 'tree/a file',
-            os.lstat('tree/a file'))
+        sha1 = self.update_entry(state, entry, 'tree/a file',
+                                 os.lstat('tree/a file'))
         self.assertEqual(expected_sha, sha1)
 
     def test_sha1provider_stat_and_sha1_used(self):
@@ -1206,8 +1256,8 @@
         state._sha1_provider = UppercaseSHA1Provider()
         # If we used the standard provider, it would look like nothing has
         # changed
-        file_ids_changed = [change[0] for change 
-                in tree.iter_changes(tree.basis_tree())]
+        file_ids_changed = [change[0] for change
+                            in tree.iter_changes(tree.basis_tree())]
         self.assertEqual(['a-file-id'], file_ids_changed)
 
 
@@ -1228,29 +1278,18 @@
         return statvalue, sha1
 
 
-class TestCompiledUpdateEntry(TestUpdateEntry):
-    """Test the pyrex implementation of _read_dirblocks"""
-
-    _test_needs_features = [CompiledDirstateHelpersFeature]
-
-    def set_update_entry(self):
-        from bzrlib._dirstate_helpers_c import update_entry
-        self.update_entry = update_entry
-
-
-class TestProcessEntryPython(test_dirstate.TestCaseWithDirState):
+class TestProcessEntry(test_dirstate.TestCaseWithDirState):
+
+    # Set by load_tests
+    _process_entry = None
 
     def setUp(self):
-        super(TestProcessEntryPython, self).setUp()
-        self.setup_process_entry()
-
-    def setup_process_entry(self):
-        from bzrlib import dirstate
+        super(TestProcessEntry, self).setUp()
         orig = dirstate._process_entry
         def cleanup():
             dirstate._process_entry = orig
         self.addCleanup(cleanup)
-        dirstate._process_entry = dirstate.ProcessEntryPython
+        dirstate._process_entry = self._process_entry
 
     def assertChangedFileIds(self, expected, tree):
         tree.lock_read()
@@ -1280,15 +1319,3 @@
         state._sha1_provider = UppercaseSHA1Provider()
         self.assertChangedFileIds(['file-id'], tree)
 
-
-class TestProcessEntryC(TestProcessEntryPython):
-    _test_needs_features = [CompiledDirstateHelpersFeature]
-
-    def setup_process_entry(self):
-        from bzrlib import _dirstate_helpers_c
-        orig = dirstate._process_entry
-        def cleanup():
-            dirstate._process_entry = orig
-        self.addCleanup(cleanup)
-        dirstate._process_entry = _dirstate_helpers_c.ProcessEntryC
-

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2009-04-17 14:03:32 +0000
+++ bzrlib/tests/test_bundle.py	2009-05-06 05:36:28 +0000
@@ -657,36 +657,60 @@
                           verbose=False)
         bundle = self.get_valid_bundle('a@cset-0-6', 'a@cset-0-7')
 
-    def test_symlink_bundle(self):
+    def _test_symlink_bundle(self, link_name, link_target, new_link_target):
+        link_id = 'link-1'
+
         self.requireFeature(tests.SymlinkFeature)
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
+
         tt = TreeTransform(self.tree1)
-        tt.new_symlink('link', tt.root, 'bar/foo', 'link-1')
+        tt.new_symlink(link_name, tt.root, link_target, link_id)
         tt.apply()
         self.tree1.commit('add symlink', rev_id='l@cset-0-1')
-        self.get_valid_bundle('null:', 'l@cset-0-1')
+        bundle = self.get_valid_bundle('null:', 'l@cset-0-1')
+        if getattr(bundle ,'revision_tree', None) is not None:
+            # Not all bundle formats supports revision_tree
+            bund_tree = bundle.revision_tree(self.b1.repository, 'l@cset-0-1')
+            self.assertEqual(link_target, bund_tree.get_symlink_target(link_id))
+
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('link-1')
+        trans_id = tt.trans_id_tree_file_id(link_id)
         tt.adjust_path('link2', tt.root, trans_id)
         tt.delete_contents(trans_id)
-        tt.create_symlink('mars', trans_id)
+        tt.create_symlink(new_link_target, trans_id)
         tt.apply()
         self.tree1.commit('rename and change symlink', rev_id='l@cset-0-2')
-        self.get_valid_bundle('l@cset-0-1', 'l@cset-0-2')
+        bundle = self.get_valid_bundle('l@cset-0-1', 'l@cset-0-2')
+        if getattr(bundle ,'revision_tree', None) is not None:
+            # Not all bundle formats supports revision_tree
+            bund_tree = bundle.revision_tree(self.b1.repository, 'l@cset-0-2')
+            self.assertEqual(new_link_target,
+                             bund_tree.get_symlink_target(link_id))
+
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('link-1')
+        trans_id = tt.trans_id_tree_file_id(link_id)
         tt.delete_contents(trans_id)
         tt.create_symlink('jupiter', trans_id)
         tt.apply()
         self.tree1.commit('just change symlink target', rev_id='l@cset-0-3')
-        self.get_valid_bundle('l@cset-0-2', 'l@cset-0-3')
+        bundle = self.get_valid_bundle('l@cset-0-2', 'l@cset-0-3')
+
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('link-1')
+        trans_id = tt.trans_id_tree_file_id(link_id)
         tt.delete_contents(trans_id)
         tt.apply()
         self.tree1.commit('Delete symlink', rev_id='l@cset-0-4')
-        self.get_valid_bundle('l@cset-0-3', 'l@cset-0-4')
+        bundle = self.get_valid_bundle('l@cset-0-3', 'l@cset-0-4')
+
+    def test_symlink_bundle(self):
+        self._test_symlink_bundle('link', 'bar/foo', 'mars')
+
+    def test_unicode_symlink_bundle(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        self._test_symlink_bundle(u'\N{Euro Sign}link',
+                                  u'bar/\N{Euro Sign}foo',
+                                  u'mars\N{Euro Sign}')
 
     def test_binary_bundle(self):
         self.tree1 = self.make_branch_and_tree('b1')

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2009-04-17 14:03:32 +0000
+++ bzrlib/tests/test_dirstate.py	2009-05-06 05:36:28 +0000
@@ -44,9 +44,35 @@
 # set_path_id  setting id when state is in memory modified
 
 
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
+        basic_tests, tests.condition_isinstance(TestCaseWithDirState))
+    tests.multiply_tests(dir_reader_tests,
+                         test_osutils.dir_reader_scenarios(), suite)
+    suite.addTest(remaining_tests)
+    return suite
+
+
 class TestCaseWithDirState(tests.TestCaseWithTransport):
     """Helper functions for creating DirState objects with various content."""
 
+    # Set by load_tests
+    _dir_reader_class = None
+    _native_to_unicode = None # Not used yet
+
+    def setUp(self):
+        tests.TestCaseWithTransport.setUp(self)
+
+        # Save platform specific info and reset it
+        cur_dir_reader = osutils._selected_dir_reader
+
+        def restore():
+            osutils._selected_dir_reader = cur_dir_reader
+        self.addCleanup(restore)
+
+        osutils._selected_dir_reader = self._dir_reader_class()
+
     def create_empty_dirstate(self):
         """Return a locked but empty dirstate"""
         state = dirstate.DirState.initialize('dirstate')
@@ -1106,24 +1132,26 @@
         state._validate()
         self.assertEqual(expected_entries, list(state._iter_entries()))
 
-    def test_add_symlink_to_root_no_parents_all_data(self):
+    def _test_add_symlink_to_root_no_parents_all_data(self, link_name, target):
         # The most trivial addition of a symlink when there are no parents and
         # its in the root and all data about the file is supplied
         # bzr doesn't support fake symlinks on windows, yet.
         self.requireFeature(tests.SymlinkFeature)
-        os.symlink('target', 'a link')
-        stat = os.lstat('a link')
+        os.symlink(target, link_name)
+        stat = os.lstat(link_name)
         expected_entries = [
             (('', '', 'TREE_ROOT'), [
              ('d', '', 0, False, dirstate.DirState.NULLSTAT), # current tree
              ]),
-            (('', 'a link', 'a link id'), [
-             ('l', 'target', 6, False, dirstate.pack_stat(stat)), # current tree
+            (('', link_name.encode('UTF-8'), 'a link id'), [
+             ('l', target.encode('UTF-8'), stat[6],
+              False, dirstate.pack_stat(stat)), # current tree
              ]),
             ]
         state = dirstate.DirState.initialize('dirstate')
         try:
-            state.add('a link', 'a link id', 'symlink', stat, 'target')
+            state.add(link_name, 'a link id', 'symlink', stat,
+                      target.encode('UTF-8'))
             # having added it, it should be in the output of iter_entries.
             self.assertEqual(expected_entries, list(state._iter_entries()))
             # saving and reloading should not affect this.
@@ -1135,6 +1163,14 @@
         self.addCleanup(state.unlock)
         self.assertEqual(expected_entries, list(state._iter_entries()))
 
+    def test_add_symlink_to_root_no_parents_all_data(self):
+        self._test_add_symlink_to_root_no_parents_all_data('a link', 'target')
+
+    def test_add_symlink_unicode_to_root_no_parents_all_data(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        self._test_add_symlink_to_root_no_parents_all_data(
+            u'\N{Euro Sign}link', u'targ\N{Euro Sign}et')
+
     def test_add_directory_and_child_no_parents_all_data(self):
         # after adding a directory, we should be able to add children to it.
         self.build_tree(['a dir/', 'a dir/a file'])
@@ -2204,14 +2240,14 @@
         self.assertIsInstance(tree_data, str)
 
     def test_unicode_symlink(self):
-        # In general, the code base doesn't support a target that contains
-        # non-ascii characters. So we just assert tha
-        inv_entry = inventory.InventoryLink('link-file-id', 'name',
+        inv_entry = inventory.InventoryLink('link-file-id',
+                                            u'nam\N{Euro Sign}e',
                                             'link-parent-id')
         inv_entry.revision = 'link-revision-id'
-        inv_entry.symlink_target = u'link-target'
-        details = self.assertDetails(('l', 'link-target', 0, False,
-                                      'link-revision-id'), inv_entry)
+        target = u'link-targ\N{Euro Sign}t'
+        inv_entry.symlink_target = target
+        self.assertDetails(('l', target.encode('UTF-8'), 0, False,
+                            'link-revision-id'), inv_entry)
 
 
 class TestSHA1Provider(tests.TestCaseInTempDir):

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2009-05-05 16:57:19 +0000
+++ bzrlib/tests/test_osutils.py	2009-05-06 05:36:28 +0000
@@ -53,7 +53,60 @@
 UTF8DirReaderFeature = _UTF8DirReaderFeature()
 
 
-class TestOSUtils(tests.TestCaseInTempDir):
+def _already_unicode(s):
+    return s
+
+
+def _fs_enc_to_unicode(s):
+    return s.decode(osutils._fs_enc)
+
+
+def _utf8_to_unicode(s):
+    return s.decode('UTF-8')
+
+
+def dir_reader_scenarios():
+    # For each dir reader we define:
+
+    # - native_to_unicode: a function converting the native_abspath as returned
+    #   by DirReader.read_dir to its unicode representation
+
+    # UnicodeDirReader is the fallback, it should be tested on all platforms.
+    scenarios = [('unicode',
+                  dict(_dir_reader_class=osutils.UnicodeDirReader,
+                       _native_to_unicode=_already_unicode))]
+    # Some DirReaders are platform specific and even there they may not be
+    # available.
+    if UTF8DirReaderFeature.available():
+        from bzrlib import _readdir_pyx
+        scenarios.append(('utf8',
+                          dict(_dir_reader_class=_readdir_pyx.UTF8DirReader,
+                               _native_to_unicode=_utf8_to_unicode)))
+
+    if test__walkdirs_win32.Win32ReadDirFeature.available():
+        try:
+            from bzrlib import _walkdirs_win32
+            # TODO: check on windows, it may be that we need to use/add
+            # safe_unicode instead of _fs_enc_to_unicode
+            scenarios.append(
+                ('win32',
+                 dict(_dir_reader_class=_walkdirs_win32.Win32ReadDir,
+                      _native_to_unicode=_fs_enc_to_unicode)))
+        except ImportError:
+            pass
+    return scenarios
+
+
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
+        basic_tests, tests.condition_isinstance(TestDirReader))
+    tests.multiply_tests(dir_reader_tests, dir_reader_scenarios(), suite)
+    suite.addTest(remaining_tests)
+    return suite
+
+
+class TestContainsWhitespace(tests.TestCase):
 
     def test_contains_whitespace(self):
         self.failUnless(osutils.contains_whitespace(u' '))
@@ -69,6 +122,9 @@
         self.failIf(osutils.contains_whitespace(u'hellothere'))
         self.failIf(osutils.contains_whitespace(u'hello\xa0there'))
 
+
+class TestRename(tests.TestCaseInTempDir):
+
     def test_fancy_rename(self):
         # This should work everywhere
         def rename(a, b):
@@ -112,6 +168,9 @@
         shape = sorted(os.listdir('.'))
         self.assertEquals(['A', 'B'], shape)
 
+
+class TestRandChars(tests.TestCase):
+
     def test_01_rand_chars_empty(self):
         result = osutils.rand_chars(0)
         self.assertEqual(result, '')
@@ -122,6 +181,9 @@
         self.assertEqual(type(result), str)
         self.assertContainsRe(result, r'^[a-z0-9]{100}$')
 
+
+class TestIsInside(tests.TestCase):
+
     def test_is_inside(self):
         is_inside = osutils.is_inside
         self.assertTrue(is_inside('src', 'src/foo.c'))
@@ -156,6 +218,9 @@
                          (['src'], 'srccontrol/foo')]:
             self.assertFalse(osutils.is_inside_or_parent_of_any(dirs, fn))
 
+
+class TestRmTree(tests.TestCaseInTempDir):
+
     def test_rmtree(self):
         # Check to remove tree with read-only files/dirs
         os.mkdir('dir')
@@ -174,6 +239,9 @@
         self.failIfExists('dir/file')
         self.failIfExists('dir')
 
+
+class TestKind(tests.TestCaseInTempDir):
+
     def test_file_kind(self):
         self.build_tree(['file', 'dir/'])
         self.assertEquals('file', osutils.file_kind('file'))
@@ -209,10 +277,15 @@
                 os.remove('socket')
 
     def test_kind_marker(self):
-        self.assertEqual(osutils.kind_marker('file'), '')
-        self.assertEqual(osutils.kind_marker('directory'), '/')
-        self.assertEqual(osutils.kind_marker('symlink'), '@')
-        self.assertEqual(osutils.kind_marker('tree-reference'), '+')
+        self.assertEqual("", osutils.kind_marker("file"))
+        self.assertEqual("/", osutils.kind_marker('directory'))
+        self.assertEqual("/", osutils.kind_marker(osutils._directory_kind))
+        self.assertEqual("@", osutils.kind_marker("symlink"))
+        self.assertEqual("+", osutils.kind_marker("tree-reference"))
+        self.assertRaises(errors.BzrError, osutils.kind_marker, "unknown")
+
+
+class TestUmask(tests.TestCaseInTempDir):
 
     def test_get_umask(self):
         if sys.platform == 'win32':
@@ -231,6 +304,9 @@
         os.umask(0027)
         self.assertEqual(0027, osutils.get_umask())
 
+
+class TestDateTime(tests.TestCase):
+
     def assertFormatedDelta(self, expected, seconds):
         """Assert osutils.format_delta formats as expected"""
         actual = osutils.format_delta(seconds)
@@ -277,6 +353,28 @@
         # Instead blackbox.test_locale should check for localized
         # dates once they do occur in output strings.
 
+    def test_local_time_offset(self):
+        """Test that local_time_offset() returns a sane value."""
+        offset = osutils.local_time_offset()
+        self.assertTrue(isinstance(offset, int))
+        # Test that the offset is no more than a eighteen hours in
+        # either direction.
+        # Time zone handling is system specific, so it is difficult to
+        # do more specific tests, but a value outside of this range is
+        # probably wrong.
+        eighteen_hours = 18 * 3600
+        self.assertTrue(-eighteen_hours < offset < eighteen_hours)
+
+    def test_local_time_offset_with_timestamp(self):
+        """Test that local_time_offset() works with a timestamp."""
+        offset = osutils.local_time_offset(1000000000.1234567)
+        self.assertTrue(isinstance(offset, int))
+        eighteen_hours = 18 * 3600
+        self.assertTrue(-eighteen_hours < offset < eighteen_hours)
+
+
+class TestLinks(tests.TestCaseInTempDir):
+
     def test_dereference_path(self):
         self.requireFeature(tests.SymlinkFeature)
         cwd = osutils.realpath('.')
@@ -325,12 +423,6 @@
             osutils.make_readonly('dangling')
             osutils.make_writable('dangling')
 
-    def test_kind_marker(self):
-        self.assertEqual("", osutils.kind_marker("file"))
-        self.assertEqual("/", osutils.kind_marker(osutils._directory_kind))
-        self.assertEqual("@", osutils.kind_marker("symlink"))
-        self.assertRaises(errors.BzrError, osutils.kind_marker, "unknown")
-
     def test_host_os_dereferences_symlinks(self):
         osutils.host_os_dereferences_symlinks()
 
@@ -815,6 +907,11 @@
 
 class TestWalkDirs(tests.TestCaseInTempDir):
 
+    def assertExpectedBlocks(self, expected, result):
+        self.assertEqual(expected,
+                         [(dirinfo, [line[0:3] for line in block])
+                          for dirinfo, block in result])
+
     def test_walkdirs(self):
         tree = [
             '.bzr',
@@ -852,14 +949,12 @@
             result.append((dirdetail, dirblock))
 
         self.assertTrue(found_bzrdir)
-        self.assertEqual(expected_dirblocks,
-            [(dirinfo, [line[0:3] for line in block]) for dirinfo, block in result])
+        self.assertExpectedBlocks(expected_dirblocks, result)
         # you can search a subdir only, with a supplied prefix.
         result = []
         for dirblock in osutils.walkdirs('./1dir', '1dir'):
             result.append(dirblock)
-        self.assertEqual(expected_dirblocks[1:],
-            [(dirinfo, [line[0:3] for line in block]) for dirinfo, block in result])
+        self.assertExpectedBlocks(expected_dirblocks[1:], result)
 
     def test_walkdirs_os_error(self):
         # <https://bugs.edge.launchpad.net/bzr/+bug/338653>
@@ -871,7 +966,7 @@
         os.mkdir("test-unreadable")
         os.chmod("test-unreadable", 0000)
         # must chmod it back so that it can be removed
-        self.addCleanup(lambda: os.chmod("test-unreadable", 0700))
+        self.addCleanup(os.chmod, "test-unreadable", 0700)
         # The error is not raised until the generator is actually evaluated.
         # (It would be ok if it happened earlier but at the moment it
         # doesn't.)
@@ -918,14 +1013,13 @@
             result.append((dirdetail, dirblock))
 
         self.assertTrue(found_bzrdir)
-        self.assertEqual(expected_dirblocks,
-            [(dirinfo, [line[0:3] for line in block]) for dirinfo, block in result])
+        self.assertExpectedBlocks(expected_dirblocks, result)
+
         # you can search a subdir only, with a supplied prefix.
         result = []
         for dirblock in osutils.walkdirs('./1dir', '1dir'):
             result.append(dirblock)
-        self.assertEqual(expected_dirblocks[1:],
-            [(dirinfo, [line[0:3] for line in block]) for dirinfo, block in result])
+        self.assertExpectedBlocks(expected_dirblocks[1:], result)
 
     def _filter_out_stat(self, result):
         """Filter out the stat value from the walkdirs result"""
@@ -946,7 +1040,7 @@
             osutils._selected_dir_reader = cur_dir_reader
         self.addCleanup(restore)
 
-    def assertReadFSDirIs(self, expected):
+    def assertDirReaderIs(self, expected):
         """Assert the right implementation for _walkdirs_utf8 is chosen."""
         # Force it to redetect
         osutils._selected_dir_reader = None
@@ -959,27 +1053,27 @@
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'UTF-8'
-        self.assertReadFSDirIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
 
     def test_force_walkdirs_utf8_fs_ascii(self):
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'US-ASCII'
-        self.assertReadFSDirIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
 
     def test_force_walkdirs_utf8_fs_ANSI(self):
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'ANSI_X3.4-1968'
-        self.assertReadFSDirIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
 
     def test_force_walkdirs_utf8_fs_latin1(self):
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'latin1'
-        self.assertReadFSDirIs(osutils.UnicodeDirReader)
+        self.assertDirReaderIs(osutils.UnicodeDirReader)
 
     def test_force_walkdirs_utf8_nt(self):
         # Disabled because the thunk of the whole walkdirs api is disabled.
@@ -987,13 +1081,13 @@
         self._save_platform_info()
         win32utils.winver = 'Windows NT'
         from bzrlib._walkdirs_win32 import Win32ReadDir
-        self.assertReadFSDirIs(Win32ReadDir)
+        self.assertDirReaderIs(Win32ReadDir)
 
     def test_force_walkdirs_utf8_98(self):
         self.requireFeature(test__walkdirs_win32.Win32ReadDirFeature)
         self._save_platform_info()
         win32utils.winver = 'Windows 98'
-        self.assertReadFSDirIs(osutils.UnicodeDirReader)
+        self.assertDirReaderIs(osutils.UnicodeDirReader)
 
     def test_unicode_walkdirs(self):
         """Walkdirs should always return unicode paths."""
@@ -1422,28 +1516,6 @@
         self.failIf('BZR_TEST_ENV_VAR' in os.environ)
 
 
-class TestLocalTimeOffset(tests.TestCase):
-
-    def test_local_time_offset(self):
-        """Test that local_time_offset() returns a sane value."""
-        offset = osutils.local_time_offset()
-        self.assertTrue(isinstance(offset, int))
-        # Test that the offset is no more than a eighteen hours in
-        # either direction.
-        # Time zone handling is system specific, so it is difficult to
-        # do more specific tests, but a value outside of this range is
-        # probably wrong.
-        eighteen_hours = 18 * 3600
-        self.assertTrue(-eighteen_hours < offset < eighteen_hours)
-
-    def test_local_time_offset_with_timestamp(self):
-        """Test that local_time_offset() works with a timestamp."""
-        offset = osutils.local_time_offset(1000000000.1234567)
-        self.assertTrue(isinstance(offset, int))
-        eighteen_hours = 18 * 3600
-        self.assertTrue(-eighteen_hours < offset < eighteen_hours)
-
-
 class TestSizeShaFile(tests.TestCaseInTempDir):
 
     def test_sha_empty(self):
@@ -1512,3 +1584,168 @@
             "Invalid regular expression in test case: '*': "
             "nothing to repeat",
             str(err))
+
+
+class TestDirReader(tests.TestCaseInTempDir):
+
+    # Set by load_tests
+    _dir_reader_class = None
+    _native_to_unicode = None
+
+    def setUp(self):
+        tests.TestCaseInTempDir.setUp(self)
+
+        # Save platform specific info and reset it
+        cur_dir_reader = osutils._selected_dir_reader
+
+        def restore():
+            osutils._selected_dir_reader = cur_dir_reader
+        self.addCleanup(restore)
+
+        osutils._selected_dir_reader = self._dir_reader_class()
+
+    def _get_ascii_tree(self):
+        tree = [
+            '0file',
+            '1dir/',
+            '1dir/0file',
+            '1dir/1dir/',
+            '2file'
+            ]
+        expected_dirblocks = [
+                (('', '.'),
+                 [('0file', '0file', 'file'),
+                  ('1dir', '1dir', 'directory'),
+                  ('2file', '2file', 'file'),
+                 ]
+                ),
+                (('1dir', './1dir'),
+                 [('1dir/0file', '0file', 'file'),
+                  ('1dir/1dir', '1dir', 'directory'),
+                 ]
+                ),
+                (('1dir/1dir', './1dir/1dir'),
+                 [
+                 ]
+                ),
+            ]
+        return tree, expected_dirblocks
+
+    def test_walk_cur_dir(self):
+        tree, expected_dirblocks = self._get_ascii_tree()
+        self.build_tree(tree)
+        result = list(osutils._walkdirs_utf8('.'))
+        # Filter out stat and abspath
+        self.assertEqual(expected_dirblocks,
+                         [(dirinfo, [line[0:3] for line in block])
+                          for dirinfo, block in result])
+
+    def test_walk_sub_dir(self):
+        tree, expected_dirblocks = self._get_ascii_tree()
+        self.build_tree(tree)
+        # you can search a subdir only, with a supplied prefix.
+        result = list(osutils._walkdirs_utf8('./1dir', '1dir'))
+        # Filter out stat and abspath
+        self.assertEqual(expected_dirblocks[1:],
+                         [(dirinfo, [line[0:3] for line in block])
+                          for dirinfo, block in result])
+
+    def _get_unicode_tree(self):
+        name0u = u'0file-\xb6'
+        name1u = u'1dir-\u062c\u0648'
+        name2u = u'2file-\u0633'
+        tree = [
+            name0u,
+            name1u + '/',
+            name1u + '/' + name0u,
+            name1u + '/' + name1u + '/',
+            name2u,
+            ]
+        name0 = name0u.encode('UTF-8')
+        name1 = name1u.encode('UTF-8')
+        name2 = name2u.encode('UTF-8')
+        expected_dirblocks = [
+                (('', '.'),
+                 [(name0, name0, 'file', './' + name0u),
+                  (name1, name1, 'directory', './' + name1u),
+                  (name2, name2, 'file', './' + name2u),
+                 ]
+                ),
+                ((name1, './' + name1u),
+                 [(name1 + '/' + name0, name0, 'file', './' + name1u
+                                                        + '/' + name0u),
+                  (name1 + '/' + name1, name1, 'directory', './' + name1u
+                                                            + '/' + name1u),
+                 ]
+                ),
+                ((name1 + '/' + name1, './' + name1u + '/' + name1u),
+                 [
+                 ]
+                ),
+            ]
+        return tree, expected_dirblocks
+
+    def _filter_out(self, raw_dirblocks):
+        """Filter out a walkdirs_utf8 result.
+
+        stat field is removed, all native paths are converted to unicode
+        """
+        filtered_dirblocks = []
+        for dirinfo, block in raw_dirblocks:
+            dirinfo = (dirinfo[0], self._native_to_unicode(dirinfo[1]))
+            details = []
+            for line in block:
+                details.append(line[0:3] + (self._native_to_unicode(line[4]), ))
+            filtered_dirblocks.append((dirinfo, details))
+        return filtered_dirblocks
+
+    def test_walk_unicode_tree(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        tree, expected_dirblocks = self._get_unicode_tree()
+        self.build_tree(tree)
+        result = list(osutils._walkdirs_utf8('.'))
+        self.assertEqual(expected_dirblocks, self._filter_out(result))
+
+    def test_symlink(self):
+        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        target = u'target\N{Euro Sign}'
+        link_name = u'l\N{Euro Sign}nk'
+        os.symlink(target, link_name)
+        target_utf8 = target.encode('UTF-8')
+        link_name_utf8 = link_name.encode('UTF-8')
+        expected_dirblocks = [
+                (('', '.'),
+                 [(link_name_utf8, link_name_utf8,
+                   'symlink', './' + link_name),],
+                 )]
+        result = list(osutils._walkdirs_utf8('.'))
+        self.assertEqual(expected_dirblocks, self._filter_out(result))
+
+
+class TestReadLink(tests.TestCaseInTempDir):
+    """Exposes os.readlink() problems and the osutils solution.
+
+    The only guarantee offered by os.readlink(), starting with 2.6, is that a
+    unicode string will be returned if a unicode string is passed.
+
+    But prior python versions failed to properly encode the passed unicode
+    string.
+    """
+    _test_needs_features = [tests.SymlinkFeature, tests.UnicodeFilenameFeature]
+
+    def setUp(self):
+        super(tests.TestCaseInTempDir, self).setUp()
+        self.link = u'l\N{Euro Sign}ink'
+        self.target = u'targe\N{Euro Sign}t'
+        os.symlink(self.target, self.link)
+
+    def test_os_readlink_link_encoding(self):
+        if sys.version_info < (2, 6):
+            self.assertRaises(UnicodeEncodeError, os.readlink, self.link)
+        else:
+            self.assertEquals(self.target,  os.readlink(self.link))
+
+    def test_os_readlink_link_decoding(self):
+        self.assertEquals(self.target.encode(osutils._fs_enc),
+                          os.readlink(self.link.encode(osutils._fs_enc)))

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2009-04-17 14:03:32 +0000
+++ bzrlib/tests/test_shelf.py	2009-05-06 05:36:28 +0000
@@ -129,45 +129,67 @@
         self.assertEqual('directory',
             creator.shelf_transform.final_kind(s_bar_trans_id))
 
+    def _test_shelve_symlink_creation(self, link_name, link_target):
+        self.requireFeature(tests.SymlinkFeature)
+        tree = self.make_branch_and_tree('.')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        tree.commit('Empty tree')
+        os.symlink(link_target, link_name)
+        tree.add(link_name, 'foo-id')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        self.assertEqual([('add file', 'foo-id', 'symlink', link_name)],
+                         list(creator.iter_shelvable()))
+        creator.shelve_creation('foo-id')
+        creator.transform()
+        s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
+        self.failIfExists(link_name)
+        limbo_name = creator.shelf_transform._limbo_name(s_trans_id)
+        self.assertEqual(link_target, osutils.readlink(limbo_name))
+        ptree = creator.shelf_transform.get_preview_tree()
+        self.assertEqual(link_target, ptree.get_symlink_target('foo-id'))
+
     def test_shelve_symlink_creation(self):
-        self.requireFeature(tests.SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        tree.lock_write()
-        self.addCleanup(tree.unlock)
-        tree.commit('Empty tree')
-        os.symlink('bar', 'foo')
-        tree.add('foo', 'foo-id')
-        creator = shelf.ShelfCreator(tree, tree.basis_tree())
-        self.addCleanup(creator.finalize)
-        self.assertEqual([('add file', 'foo-id', 'symlink', 'foo')],
-                         list(creator.iter_shelvable()))
-        creator.shelve_creation('foo-id')
-        creator.transform()
-        s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
-        self.failIfExists('foo')
-        limbo_name = creator.shelf_transform._limbo_name(s_trans_id)
-        self.assertEqual('bar', os.readlink(limbo_name))
-
-    def test_shelve_symlink_target_change(self):
-        self.requireFeature(tests.SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        tree.lock_write()
-        self.addCleanup(tree.unlock)
-        os.symlink('bar', 'foo')
-        tree.add('foo', 'foo-id')
+        self._test_shelve_symlink_creation('foo', 'bar')
+
+    def test_shelve_unicode_symlink_creation(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        self._test_shelve_symlink_creation(u'fo\N{Euro Sign}o',
+                                           u'b\N{Euro Sign}ar')
+
+    def _test_shelve_symlink_target_change(self, link_name,
+                                           old_target, new_target):
+        self.requireFeature(tests.SymlinkFeature)
+        tree = self.make_branch_and_tree('.')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        os.symlink(old_target, link_name)
+        tree.add(link_name, 'foo-id')
         tree.commit("commit symlink")
-        os.unlink("foo")
-        os.symlink('baz', 'foo')
+        os.unlink(link_name)
+        os.symlink(new_target, link_name)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
-        self.assertEqual([('modify target', 'foo-id', 'foo', 'bar', 'baz')],
+        self.assertEqual([('modify target', 'foo-id', link_name,
+                           old_target, new_target)],
                          list(creator.iter_shelvable()))
         creator.shelve_modify_target('foo-id')
         creator.transform()
-        self.assertEqual('bar', os.readlink('foo'))
+        self.assertEqual(old_target, osutils.readlink(link_name))
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
         limbo_name = creator.shelf_transform._limbo_name(s_trans_id)
-        self.assertEqual('baz', os.readlink(limbo_name))
+        self.assertEqual(new_target, osutils.readlink(limbo_name))
+        ptree = creator.shelf_transform.get_preview_tree()
+        self.assertEqual(new_target, ptree.get_symlink_target('foo-id'))
+
+    def test_shelve_symlink_target_change(self):
+        self._test_shelve_symlink_target_change('foo', 'bar', 'baz')
+
+    def test_shelve_unicode_symlink_target_change(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        self._test_shelve_symlink_target_change(
+            u'fo\N{Euro Sign}o', u'b\N{Euro Sign}ar', u'b\N{Euro Sign}az')
 
     def test_shelve_creation_no_contents(self):
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/test_tag.py'
--- bzrlib/tests/test_tag.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_tag.py	2009-05-06 05:36:28 +0000
@@ -50,6 +50,24 @@
         self.assertEqual(store._deserialize_tag_dict(packed), td)
 
 
+class TestTagRevisionRenames(TestCaseWithTransport):
+
+    def make_branch_supporting_tags(self, relpath):
+        return self.make_branch(relpath, format='dirstate-tags')
+
+    def test_simple(self):
+        store = self.make_branch_supporting_tags('a').tags
+        store.set_tag("foo", "myoldrevid")
+        store.rename_revisions({"myoldrevid": "mynewrevid"})
+        self.assertEquals({"foo": "mynewrevid"}, store.get_tag_dict())
+
+    def test_unknown_ignored(self):
+        store = self.make_branch_supporting_tags('a').tags
+        store.set_tag("foo", "myoldrevid")
+        store.rename_revisions({"anotherrevid": "mynewrevid"})
+        self.assertEquals({"foo": "myoldrevid"}, store.get_tag_dict())
+
+
 class TestTagMerging(TestCaseWithTransport):
 
     def make_knit_branch(self, relpath):

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2009-04-06 11:41:40 +0000
+++ bzrlib/tests/test_transform.py	2009-05-06 05:36:28 +0000
@@ -525,26 +525,42 @@
         resolve_conflicts(replace)
         replace.apply()
 
-    def test_symlinks(self):
+    def _test_symlinks(self, link_name1,link_target1,
+                       link_name2, link_target2):
+
+        def ozpath(p): return 'oz/' + p
+
         self.requireFeature(SymlinkFeature)
-        transform,root = self.get_transform()
+        transform, root = self.get_transform()
         oz_id = transform.new_directory('oz', root, 'oz-id')
-        wizard = transform.new_symlink('wizard', oz_id, 'wizard-target',
+        wizard = transform.new_symlink(link_name1, oz_id, link_target1,
                                        'wizard-id')
-        wiz_id = transform.create_path('wizard2', oz_id)
-        transform.create_symlink('behind_curtain', wiz_id)
+        wiz_id = transform.create_path(link_name2, oz_id)
+        transform.create_symlink(link_target2, wiz_id)
         transform.version_file('wiz-id2', wiz_id)
         transform.set_executability(True, wiz_id)
         self.assertEqual(transform.find_conflicts(),
                          [('non-file executability', wiz_id)])
         transform.set_executability(None, wiz_id)
         transform.apply()
-        self.assertEqual(self.wt.path2id('oz/wizard'), 'wizard-id')
-        self.assertEqual(file_kind(self.wt.abspath('oz/wizard')), 'symlink')
-        self.assertEqual(os.readlink(self.wt.abspath('oz/wizard2')),
-                         'behind_curtain')
-        self.assertEqual(os.readlink(self.wt.abspath('oz/wizard')),
-                         'wizard-target')
+        self.assertEqual(self.wt.path2id(ozpath(link_name1)), 'wizard-id')
+        self.assertEqual('symlink',
+                         file_kind(self.wt.abspath(ozpath(link_name1))))
+        self.assertEqual(link_target2,
+                         osutils.readlink(self.wt.abspath(ozpath(link_name2))))
+        self.assertEqual(link_target1,
+                         osutils.readlink(self.wt.abspath(ozpath(link_name1))))
+
+    def test_symlinks(self):
+        self._test_symlinks('wizard', 'wizard-target',
+                            'wizard2', 'behind_curtain')
+
+    def test_symlinks_unicode(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        self._test_symlinks(u'\N{Euro Sign}wizard',
+                            u'wizard-targ\N{Euro Sign}t',
+                            u'\N{Euro Sign}wizard2',
+                            u'b\N{Euro Sign}hind_curtain')
 
     def test_unable_create_symlink(self):
         def tt_helper():
@@ -1841,8 +1857,8 @@
         self.assertTrue(source.is_executable('file1-id'))
 
     def test_case_insensitive_build_tree_inventory(self):
-        if (not tests.CaseInsensitiveFilesystemFeature.available()
-            or not tests.CaseInsCasePresFilenameFeature.available()):
+        if (tests.CaseInsensitiveFilesystemFeature.available()
+            or tests.CaseInsCasePresFilenameFeature.available()):
             raise tests.UnavailableFeature('Fully case sensitive filesystem')
         source = self.make_branch_and_tree('source')
         self.build_tree(['source/file', 'source/FILE'])
@@ -2688,10 +2704,11 @@
         self.assertSerializesTo(self.symlink_creation_records(), tt)
 
     def test_deserialize_symlink_creation(self):
+        self.requireFeature(tests.SymlinkFeature)
         tt = self.get_preview()
         tt.deserialize(iter(self.symlink_creation_records()))
-        # XXX readlink should be returning unicode, not utf-8
-        foo_content = os.readlink(tt._limbo_name('new-1')).decode('utf-8')
+        abspath = tt._limbo_name('new-1')
+        foo_content = osutils.readlink(abspath)
         self.assertEqual(u'bar\u1234', foo_content)
 
     def make_destruction_preview(self):

=== modified file 'bzrlib/tests/tree_implementations/test_get_symlink_target.py'
--- bzrlib/tests/tree_implementations/test_get_symlink_target.py	2009-04-11 16:06:53 +0000
+++ bzrlib/tests/tree_implementations/test_get_symlink_target.py	2009-05-06 05:36:28 +0000
@@ -53,9 +53,9 @@
         tree = self.make_branch_and_tree('tree')
         target = u'targ\N{Euro Sign}t'
         os.symlink(target,  u'tree/\u03b2_link'.encode(osutils._fs_enc))
-        tree.add([u'\u03b2_link'], ['unicode-link-id'])
+        tree.add([u'\u03b2_link'], ['link-id'])
         tree.lock_read()
         self.addCleanup(tree.unlock)
-        actual = tree.get_symlink_target(u'unicode-link-id')
+        actual = tree.get_symlink_target('link-id')
         self.assertEqual(target, actual)
 

=== modified file 'bzrlib/tests/workingtree_implementations/test_parents.py'
--- bzrlib/tests/workingtree_implementations/test_parents.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/workingtree_implementations/test_parents.py	2009-05-06 05:36:28 +0000
@@ -237,12 +237,20 @@
         # The link points to a file whose name is an omega
         # U+03A9 GREEK CAPITAL LETTER OMEGA
         # UTF-8: ce a9  UTF-16BE: 03a9  Decimal: &#937;
-        os.symlink(u'\u03a9','tree1/link_name')
-        tree.add(['link_name'],['link-id'])
+        target = u'\u03a9'
+        link_name = u'\N{Euro Sign}link'
+        os.symlink(target, 'tree1/' + link_name)
+        tree.add([link_name],['link-id'])
 
         revision1 = tree.commit('added a link to a Unicode target')
         revision2 = tree.commit('this revision will be discarded')
         tree.set_parent_ids([revision1])
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        # Check that the symlink target is safely round-tripped in the trees.
+        self.assertEqual(target, tree.get_symlink_target('link-id'))
+        basis = tree.basis_tree()
+        self.assertEqual(target, basis.get_symlink_target('link-id'))
 
 
 class TestAddParent(TestParents):

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2009-04-02 04:12:11 +0000
+++ bzrlib/transform.py	2009-05-06 05:36:28 +0000
@@ -1910,7 +1910,7 @@
             return self._transform._tree.get_symlink_target(file_id)
         trans_id = self._transform.trans_id_file_id(file_id)
         name = self._transform._limbo_name(trans_id)
-        return os.readlink(name)
+        return osutils.readlink(name)
 
     def walkdirs(self, prefix=''):
         pending = [self._transform.root]

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2009-04-04 02:50:01 +0000
+++ bzrlib/workingtree.py	2009-05-06 05:36:28 +0000
@@ -749,9 +749,8 @@
                 kind = 'tree-reference'
             return kind, None, None, None
         elif kind == 'symlink':
-            return ('symlink', None, None,
-                    os.readlink(abspath.encode(osutils._fs_enc)
-                                ).decode(osutils._fs_enc))
+            target = osutils.readlink(abspath)
+            return ('symlink', None, None, target)
         else:
             return (kind, None, None, None)
 
@@ -973,8 +972,9 @@
         return file_id
 
     def get_symlink_target(self, file_id):
-        return os.readlink(self.id2abspath(file_id).encode(osutils._fs_enc)
-            ).decode(osutils._fs_enc)
+        abspath = self.id2abspath(file_id)
+        target = osutils.readlink(abspath)
+        return target
 
     @needs_write_lock
     def subsume(self, other_tree):

